// dibview.cpp : implementation of the CDibView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "diblook.h"

#include "dibdoc.h"
#include "dibview.h"
#include "dibapi.h"
#include "mainfrm.h"

#include "jpegdib.h"

extern "C" {
#include "jpeglib.h"
#include "transupp.h"
}

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDibView

IMPLEMENT_DYNCREATE(CDibView, CScrollView)

BEGIN_MESSAGE_MAP(CDibView, CScrollView)
	//{{AFX_MSG_MAP(CDibView)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_SELECT_ALL, OnEditSelectAll)
	ON_COMMAND(ID_INDICATOR_POS, OnIndicatorPos)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_POS, OnUpdateIndicatorPos)
	ON_MESSAGE(WM_DOREALIZE, OnDoRealize)
	ON_COMMAND(ID_POPUP1_MOVEFRAME, OnPopup1MoveFrame)
	ON_COMMAND(ID_POPUP1_DEFINEFRAME, OnPopup1DefineFrame)
	ON_COMMAND(ID_POPUP1_SAVEFRAMEAS, OnPopup1SaveFrameAs)
	ON_COMMAND(ID_VIEW_ZOOM1, OnViewZoom1)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM1, OnUpdateViewZoom1)
	ON_COMMAND(ID_VIEW_ZOOM2, OnViewZoom2)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM2, OnUpdateViewZoom2)
	ON_COMMAND(ID_VIEW_ZOOM3, OnViewZoom3)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM3, OnUpdateViewZoom3)
	ON_COMMAND(ID_VIEW_ZOOM4, OnViewZoom4)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM4, OnUpdateViewZoom4)
	ON_COMMAND(ID_VIEW_ZOOM5, OnViewZoom5)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM5, OnUpdateViewZoom5)
	ON_COMMAND(ID_VIEW_ZOOM6, OnViewZoom6)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM6, OnUpdateViewZoom6)
	ON_COMMAND(ID_VIEW_ZOOM7, OnViewZoom7)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM7, OnUpdateViewZoom7)
	ON_COMMAND(ID_VIEW_ZOOM8, OnViewZoom8)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM8, OnUpdateViewZoom8)
	ON_COMMAND(ID_VIEW_ZOOM9, OnViewZoom9)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM9, OnUpdateViewZoom9)
	ON_COMMAND(ID_VIEW_ZOOM10, OnViewZoom10)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM10, OnUpdateViewZoom10)
	ON_COMMAND(ID_VIEW_ZOOM11, OnViewZoom11)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM11, OnUpdateViewZoom11)
	ON_COMMAND(ID_VIEW_ZOOM12, OnViewZoom12)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM12, OnUpdateViewZoom12)
	ON_COMMAND(ID_VIEW_ZOOM13, OnViewZoom13)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM13, OnUpdateViewZoom13)
	ON_COMMAND(ID_VIEW_ZOOM14, OnViewZoom14)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM14, OnUpdateViewZoom14)
	ON_COMMAND(ID_VIEW_ZOOM15, OnViewZoom15)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM15, OnUpdateViewZoom15)
	ON_COMMAND(ID_VIEW_ZOOM16, OnViewZoom16)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZOOM16, OnUpdateViewZoom16)
	ON_UPDATE_COMMAND_UI(ID_TOOLBAR_ZOOM, OnUpdateToolbarZoom)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP

	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDibView construction/destruction

CDibView::CDibView()
{
	m_left = 0;
	m_top = 0;
	m_right = 0;
	m_bottom = 0;
	m_flag = 0;
	m_x = 0;
	m_y = 0;
	m_zoomval = 8;
}

CDibView::~CDibView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CDibView drawing

void CDibView::OnDraw(CDC* pDC)
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (pDoc->GetHDIB() == NULL)
	{
		CRect rcDisp;
		GetClientRect(rcDisp);
		rcDisp += GetDeviceScrollPosition();
		pDC->FillSolidRect(rcDisp, 0x00808080L);
		return;
	}

	CRect rcDIB;
	if (myapp->m_show_cropmask)
	{
		switch (pDoc->m_scale & 7)
		{
		case 0:
			rcDIB.left = m_left;
			rcDIB.top = m_top;
			rcDIB.right = pDoc->m_image_width - m_right;
			rcDIB.bottom = pDoc->m_image_height - m_bottom; break;
		case 1:
			rcDIB.left = (m_left + 7) >> 3;
			rcDIB.top = (m_top + 7) >> 3;
			rcDIB.right = (pDoc->m_image_width - m_right + 7) >> 3;
			rcDIB.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
		case 2:
			rcDIB.left = (m_left + 3) >> 2;
			rcDIB.top = (m_top + 3) >> 2;
			rcDIB.right = (pDoc->m_image_width - m_right + 3) >> 2;
			rcDIB.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
		case 3:
			rcDIB.left = (m_left * 3 + 7) >> 3;
			rcDIB.top = (m_top * 3 + 7) >> 3;
			rcDIB.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
			rcDIB.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
		case 4:
			rcDIB.left = (m_left + 1) >> 1;
			rcDIB.top = (m_top + 1) >> 1;
			rcDIB.right = (pDoc->m_image_width - m_right + 1) >> 1;
			rcDIB.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
		case 5:
			rcDIB.left = (m_left * 5 + 7) >> 3;
			rcDIB.top = (m_top * 5 + 7) >> 3;
			rcDIB.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
			rcDIB.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
		case 6:
			rcDIB.left = (m_left * 3 + 3) >> 2;
			rcDIB.top = (m_top * 3 + 3) >> 2;
			rcDIB.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
			rcDIB.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
		case 7:
			rcDIB.left = (m_left * 7 + 7) >> 3;
			rcDIB.top = (m_top * 7 + 7) >> 3;
			rcDIB.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
			rcDIB.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
		}
		if (pDoc->m_scale > 8)
		{
			rcDIB.left += m_left;
			rcDIB.top += m_top;
			rcDIB.right += pDoc->m_image_width - m_right;
			rcDIB.bottom += pDoc->m_image_height - m_bottom;
		}
	}
	else
	{
		rcDIB.left = 0;
		rcDIB.top = 0;
		rcDIB.right = pDoc->GetDocSize().cx;
		rcDIB.bottom = pDoc->GetDocSize().cy;
	}
	CRect rcDest;
	if (pDC->IsPrinting())   // printer DC
	{
		// get size of printer page (in pixels)
		int cxPage = pDC->GetDeviceCaps(HORZRES);
		int cyPage = pDC->GetDeviceCaps(VERTRES);
		// get printer pixels per inch
		int cxInch = pDC->GetDeviceCaps(LOGPIXELSX);
		int cyInch = pDC->GetDeviceCaps(LOGPIXELSY);

		int scale = pDoc->m_scale;
		if (scale == 8) scale = m_zoomval;

		//
		// Best Fit case -- create a rectangle which preserves
		// the DIB's aspect ratio, and fills the page horizontally.
		//
		// The formula in the "->bottom" field below calculates the Y
		// position of the printed bitmap, based on the size of the
		// bitmap, the width of the page, and the relative size of
		// a printed pixel (cyInch / cxInch).
		//
		rcDest.left = 0;
		rcDest.top = 0;
		switch (scale)
		if (scale)
		{
		case 1:
			rcDest.right = (cxPage + 7) >> 3;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				+ 7) >> 3; break;
		case 2:
			rcDest.right = (cxPage + 3) >> 2;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				+ 3) >> 2; break;
		case 3:
			rcDest.right = (cxPage * 3 + 7) >> 3;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				* 3 + 7) >> 3; break;
		case 4:
			rcDest.right = (cxPage + 1) >> 1;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				+ 1) >> 1; break;
		case 5:
			rcDest.right = (cxPage * 5 + 7) >> 3;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				* 5 + 7) >> 3; break;
		case 6:
			rcDest.right = (cxPage * 3 + 3) >> 2;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				* 3 + 3) >> 2; break;
		case 7:
			rcDest.right = (cxPage * 7 + 7) >> 3;
			rcDest.bottom = ((int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch))
				* 7 + 7) >> 3; break;
		default:
			rcDest.right = cxPage;
			rcDest.bottom = (int)(((double)(rcDIB.bottom - rcDIB.top) * cxPage *
				cyInch) / ((double)(rcDIB.right - rcDIB.left) * cxInch));
		}
	}
	else   // not printer DC
	{
		switch (m_zoomval & 7)
		{
		case 0:
			rcDest = rcDIB; break;
		case 1:
			rcDest.left = (rcDIB.left + 7) >> 3;
			rcDest.top = (rcDIB.top + 7) >> 3;
			rcDest.right = (rcDIB.right + 7) >> 3;
			rcDest.bottom = (rcDIB.bottom + 7) >> 3; break;
		case 2:
			rcDest.left = (rcDIB.left + 3) >> 2;
			rcDest.top = (rcDIB.top + 3) >> 2;
			rcDest.right = (rcDIB.right + 3) >> 2;
			rcDest.bottom = (rcDIB.bottom + 3) >> 2; break;
		case 3:
			rcDest.left = (rcDIB.left * 3 + 7) >> 3;
			rcDest.top = (rcDIB.top * 3 + 7) >> 3;
			rcDest.right = (rcDIB.right * 3 + 7) >> 3;
			rcDest.bottom = (rcDIB.bottom * 3 + 7) >> 3; break;
		case 4:
			rcDest.left = (rcDIB.left + 1) >> 1;
			rcDest.top = (rcDIB.top + 1) >> 1;
			rcDest.right = (rcDIB.right + 1) >> 1;
			rcDest.bottom = (rcDIB.bottom + 1) >> 1; break;
		case 5:
			rcDest.left = (rcDIB.left * 5 + 7) >> 3;
			rcDest.top = (rcDIB.top * 5 + 7) >> 3;
			rcDest.right = (rcDIB.right * 5 + 7) >> 3;
			rcDest.bottom = (rcDIB.bottom * 5 + 7) >> 3; break;
		case 6:
			rcDest.left = (rcDIB.left * 3 + 3) >> 2;
			rcDest.top = (rcDIB.top * 3 + 3) >> 2;
			rcDest.right = (rcDIB.right * 3 + 3) >> 2;
			rcDest.bottom = (rcDIB.bottom * 3 + 3) >> 2; break;
		case 7:
			rcDest.left = (rcDIB.left * 7 + 7) >> 3;
			rcDest.top = (rcDIB.top * 7 + 7) >> 3;
			rcDest.right = (rcDIB.right * 7 + 7) >> 3;
			rcDest.bottom = (rcDIB.bottom * 7 + 7) >> 3; break;
		}
		if (m_zoomval > 8)
		{
			rcDest.left += rcDIB.left;
			rcDest.top += rcDIB.top;
			rcDest.right += rcDIB.right;
			rcDest.bottom += rcDIB.bottom;
		}
		CRect rcDisp;
		GetClientRect(rcDisp);
		rcDisp += GetDeviceScrollPosition();
		if (rcDisp.top < rcDest.top)
		{
			int temp = rcDisp.bottom;
			rcDisp.bottom = rcDest.top;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
			rcDisp.top = rcDest.top;
			rcDisp.bottom = temp;
		}
		if (rcDisp.bottom > rcDest.bottom)
		{
			int temp = rcDisp.top;
			rcDisp.top = rcDest.bottom;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
			rcDisp.top = temp;
			rcDisp.bottom = rcDest.bottom;
		}
		if (rcDisp.left < rcDest.left)
		{
			int temp = rcDisp.right;
			rcDisp.right = rcDest.left;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
			rcDisp.right = temp;
		}
		if (rcDisp.right > rcDest.right)
		{
			rcDisp.left = rcDest.right;
			pDC->FillSolidRect(rcDisp, 0x00808080L);
		}
	}
	::PaintDIB(pDC->m_hDC, &rcDest, pDoc->GetHDIB(),
		&rcDIB, pDoc->GetDocPalette());

	if (pDC->IsPrinting()) return;

	int scale = pDoc->m_scale;
	if (scale == 8) scale = m_zoomval;

	if (myapp->m_show_blockgrid && scale >= 8)
	{
		CRect rect;
		GetClientRect(rect);
		rect += GetDeviceScrollPosition();
		if (rect.left < rcDest.left) rect.left = rcDest.left;
		if (rect.top < rcDest.top) rect.top = rcDest.top;
		if (rect.right > rcDest.right) rect.right = rcDest.right;
		if (rect.bottom > rcDest.bottom) rect.bottom = rcDest.bottom;
#if 0
		LOGBRUSH mybrush;
		mybrush.lbStyle = BS_SOLID;
		mybrush.lbColor = (COLORREF)0x00000000L;
		DWORD mystyle[2];
		mystyle[0] = 1;
		mystyle[1] = 1;
		CPen mypen(PS_USERSTYLE | PS_COSMETIC, 1, &mybrush, 2, mystyle);
#else
		// CPen mypen(PS_ALTERNATE | PS_COSMETIC, 1, (COLORREF)0x00000000L);
		CPen mypen(0, 0, (COLORREF)0x00555555L);
#endif
		CPen *pold = pDC->SelectObject(&mypen);
		// pDC->SetBkMode(TRANSPARENT);

		int incr = (pDoc->m_MCUwidth * scale) >> 3;
		for (int i = 0; i < rect.left; i += incr) ;
		if (i == rcDest.left) i += incr;
		for (; i < rect.right; i += incr)
		{
		  pDC->MoveTo(i, rect.top);
		  pDC->LineTo(i, rect.bottom);
		}

		incr = (pDoc->m_MCUheight * scale) >> 3;
		for (i = 0; i < rect.top; i += incr) ;
		if (i == rcDest.top) i += incr;
		for (; i < rect.bottom; i += incr)
		{
		  pDC->MoveTo(rect.left, i);
		  pDC->LineTo(rect.right, i);
		}

		pDC->SelectObject(pold);
	}

	pDoc->m_crop_xoffset = m_left;
	pDoc->m_crop_yoffset = m_top;
	pDoc->m_crop_width = pDoc->m_image_width - m_right - m_left;
	pDoc->m_crop_height = pDoc->m_image_height - m_bottom - m_top;

	if (m_left || m_top || m_right || m_bottom)
	{
		CPen mypen(PS_DOT, 0, (COLORREF)0x00000000L);
		CPen *pold = pDC->SelectObject(&mypen);
		pDC->SetBkColor((COLORREF)0x00FFFFFFL);
		if (myapp->m_show_cropmask)
		{
			int x, y;
			switch (scale & 7)
			{
			case 0:
				x = pDoc->m_image_width;
				y = pDoc->m_image_height; break;
			case 1:
				x = (pDoc->m_image_width + 7) >> 3;
				y = (pDoc->m_image_height + 7) >> 3; break;
			case 2:
				x = (pDoc->m_image_width + 3) >> 2;
				y = (pDoc->m_image_height + 3) >> 2; break;
			case 3:
				x = (pDoc->m_image_width * 3 + 7) >> 3;
				y = (pDoc->m_image_height * 3 + 7) >> 3; break;
			case 4:
				x = (pDoc->m_image_width + 1) >> 1;
				y = (pDoc->m_image_height + 1) >> 1; break;
			case 5:
				x = (pDoc->m_image_width * 5 + 7) >> 3;
				y = (pDoc->m_image_height * 5 + 7) >> 3; break;
			case 6:
				x = (pDoc->m_image_width * 3 + 3) >> 2;
				y = (pDoc->m_image_height * 3 + 3) >> 2; break;
			case 7:
				x = (pDoc->m_image_width * 7 + 7) >> 3;
				y = (pDoc->m_image_height * 7 + 7) >> 3; break;
			}
			if (scale > 8)
			{
				x += pDoc->m_image_width;
				y += pDoc->m_image_height;
			}
			pDC->MoveTo(0, y);
			pDC->LineTo(x, y);
			pDC->LineTo(x, 0);
		}
		else
		{
			int x1, y1, x2, y2;
			switch (scale & 7)
			{
			case 0:
				x1 = m_left;
				y1 = m_top;
				x2 = pDoc->m_image_width - m_right;
				y2 = pDoc->m_image_height - m_bottom; break;
			case 1:
				x1 = (m_left + 7) >> 3;
				y1 = (m_top + 7) >> 3;
				x2 = (pDoc->m_image_width - m_right + 7) >> 3;
				y2 = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
			case 2:
				x1 = (m_left + 3) >> 2;
				y1 = (m_top + 3) >> 2;
				x2 = (pDoc->m_image_width - m_right + 3) >> 2;
				y2 = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
			case 3:
				x1 = (m_left * 3 + 7) >> 3;
				y1 = (m_top * 3 + 7) >> 3;
				x2 = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
				y2 = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
			case 4:
				x1 = (m_left + 1) >> 1;
				y1 = (m_top + 1) >> 1;
				x2 = (pDoc->m_image_width - m_right + 1) >> 1;
				y2 = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
			case 5:
				x1 = (m_left * 5 + 7) >> 3;
				y1 = (m_top * 5 + 7) >> 3;
				x2 = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
				y2 = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
			case 6:
				x1 = (m_left * 3 + 3) >> 2;
				y1 = (m_top * 3 + 3) >> 2;
				x2 = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
				y2 = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
			case 7:
				x1 = (m_left * 7 + 7) >> 3;
				y1 = (m_top * 7 + 7) >> 3;
				x2 = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
				y2 = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
			}
			if (scale > 8)
			{
				x1 += m_left;
				y1 += m_top;
				x2 += pDoc->m_image_width - m_right;
				y2 += pDoc->m_image_height - m_bottom;
			}
			x2--;
			y2--;
			pDC->MoveTo(x1, y1);
			pDC->LineTo(x2, y1);
			pDC->LineTo(x2, y2);
			pDC->LineTo(x1, y2);
			pDC->LineTo(x1, y1);
		}
		pDC->SelectObject(pold);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDibView printing

BOOL CDibView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CDibView commands


LRESULT CDibView::OnDoRealize(WPARAM wParam, LPARAM)
{
	ASSERT(wParam != NULL);
	CDibDoc* pDoc = GetDocument();
	if (pDoc->GetHDIB() == NULL)
		return 0L;  // must be a new document


	CPalette* pPal = pDoc->GetDocPalette();
	if (pPal != NULL)
	{
		CMainFrame* pAppFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;
		ASSERT_KINDOF(CMainFrame, pAppFrame);

		CClientDC appDC(pAppFrame);
		// All views but one should be a background palette.
		// wParam contains a handle to the active view, so the SelectPalette
		// bForceBackground flag is FALSE only if wParam == m_hWnd (this view)
		CPalette* oldPalette = appDC.SelectPalette(pPal, ((HWND)wParam) != m_hWnd);

		if (oldPalette != NULL)
		{
			UINT nColorsChanged = appDC.RealizePalette();
			if (nColorsChanged > 0)
				pDoc->UpdateAllViews(NULL);
			appDC.SelectPalette(oldPalette, TRUE);
		}
		else
		{
			TRACE0("\tSelectPalette failed in CDibView::OnPaletteChanged\n");
		}
	}
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);

	return 0L;
}

static unsigned char screen_set[3][256];

typedef short FSERROR;		/* 16 bits should be enough */
typedef int LOCFSERROR;		/* use 'int' for calculation temps */

typedef struct
{
	unsigned char *colorset;
	FSERROR *fserrors;
}
FSBUF;

FSBUF *fs2_init(int width, int nc)
{
  FSBUF *fs;
  FSERROR *p;
  int i;

  fs = (FSBUF *)
    malloc(sizeof(FSBUF) * nc + ((size_t)width + 2) * sizeof(FSERROR) * nc);
  if (fs == 0) return fs;

  for (i = 0; i < nc; i++) fs[i].colorset = screen_set[i];

  p = (FSERROR *)(fs + nc);
  memset(p, 0, ((size_t)width + 2) * sizeof(FSERROR) * nc);

  for (i = 0; i < nc; i++) fs[i].fserrors = p++;

  return fs;
}

int fs2_dither(FSBUF *fs, unsigned char *ptr, int nc, int num_rows, int num_cols, int num_scan)
{
  int abs_nc, ci, row, col;
  LOCFSERROR delta, cur, belowerr, bpreverr;
  unsigned char *dataptr, *colsetptr;
  FSERROR *errorptr;

  if ((abs_nc = nc) < 0) abs_nc = -abs_nc;
  num_scan -= abs_nc;
  for (row = 0; row < num_rows; row++) {
    for (ci = 0; ci < abs_nc; ci++, ptr++) {
      dataptr = ptr;
      colsetptr = fs[ci].colorset;
      errorptr = fs[ci].fserrors;
      if (nc < 0) {
		dataptr += (num_cols - 1) * abs_nc;
		errorptr += (num_cols + 1) * abs_nc;
      }
      cur = belowerr = bpreverr = 0;
      for (col = 0; col < num_cols; col++) {
		cur += errorptr[nc];
		cur += 8; cur >>= 4;
		if ((cur += *dataptr) < 0) cur = 0;
		else if (cur > 255) cur = 255;
		*dataptr = colsetptr[cur];
		cur -= colsetptr[cur];
		delta = cur << 1; cur += delta;
		bpreverr += cur; cur += delta;
		belowerr += cur; cur += delta;
		errorptr[0] = (FSERROR)bpreverr;
		bpreverr = belowerr;
		belowerr = delta >> 1;
		dataptr += nc;
		errorptr += nc;
      }
      errorptr[0] = (FSERROR)bpreverr;
    }
	ptr += num_scan;
    nc = -nc;
  }
  return nc;
}

void CDibView::SmartDither()
{
	static int init_flag = 0;
	CDC *pDC = GetDC();
	if (((CDibLookApp *)AfxGetApp())->m_dither &&
		pDC->GetDeviceCaps(BITSPIXEL) == 16 &&
		GetDocument()->m_dithered == 0)
	{
		if (init_flag == 0)
		{
			init_flag = 1;
			for (int i = 0; i < 256; i++)
			{
				screen_set[2][i] = (unsigned char)
					(pDC->GetNearestColor((COLORREF)i) & 255);
				screen_set[1][i] = (unsigned char)
					((pDC->GetNearestColor((COLORREF)(i << 8)) >> 8) & 255);
				screen_set[0][i] = (unsigned char)
					((pDC->GetNearestColor((COLORREF)(i << 16)) >> 16) & 255);
			}
		}
		HDIB hDib = GetDocument()->GetHDIB();
		if (hDib)
		{
			LPBITMAPINFOHEADER lpDIB;
			int DIBLineWidth;
			int DIBScanWidth;
			unsigned char *lpBits;

			lpDIB = (LPBITMAPINFOHEADER)GlobalLock(hDib);

			DIBLineWidth = lpDIB->biWidth * lpDIB->biBitCount / 8;
			DIBScanWidth = (DIBLineWidth + 3) & (-4);

			lpBits = (unsigned char *) FindDIBBits((LPSTR) lpDIB);

			FSBUF *fs = fs2_init(lpDIB->biWidth, lpDIB->biBitCount / 8);
			if (fs) {
				fs2_dither(fs, lpBits, lpDIB->biBitCount / 8,
						   lpDIB->biHeight, lpDIB->biWidth,
						   DIBScanWidth);
				free(fs);
				GetDocument()->m_dithered = 1;
			}

			GlobalUnlock(hDib);
		}
	}
}

void CDibView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	if (GetDocument() == NULL) return;

	SmartDither();


    SetScrollSizes(MM_TEXT,	GetDocument()->GetDocSize());
    ResizeParentToFit ();

	m_left = 0;
	m_top = 0;
	m_right = 0;
	m_bottom = 0;
	if (((CDibLookApp *)AfxGetApp())->m_trim)
	{
		switch (GetDocument()->m_transform)
		{
		case 1:
		case 5:
			if (GetDocument()->m_image_width > GetDocument()->m_MCUwidth)
				m_right = GetDocument()->m_image_width % GetDocument()->m_MCUwidth;
			break;
		case 2:
		case 7:
			if (GetDocument()->m_image_height > GetDocument()->m_MCUheight)
				m_bottom = GetDocument()->m_image_height % GetDocument()->m_MCUheight;
			break;
		case 4:
		case 6:
			if (GetDocument()->m_image_width > GetDocument()->m_MCUwidth)
				m_right = GetDocument()->m_image_width % GetDocument()->m_MCUwidth;
			if (GetDocument()->m_image_height > GetDocument()->m_MCUheight)
				m_bottom = GetDocument()->m_image_height % GetDocument()->m_MCUheight;
			break;
		}
	}
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);
}

void CDibView::OnUpdate(CView* pSender,
						LPARAM lHint,
						CObject* pHint)
{
	if ((int)lHint >= 4 && (int)lHint <= 19)
	{
		m_zoomval = (int)lHint - 3;
	}
	CScrollView::OnUpdate(pSender, lHint, pHint);
    if (lHint == 0 || GetDocument() == NULL) return;
	if (lHint == 2 || ((int)lHint >= 4 && (int)lHint <= 19))
	{
		if (m_zoomval == 8)
		{
			SetScrollSizes(MM_TEXT,	GetDocument()->GetDocSize());
		}
		else
		{
			int xoffset, yoffset;
			switch (m_zoomval & 7)
			{
			case 0:
				xoffset = GetDocument()->GetDocSize().cx;
				yoffset = GetDocument()->GetDocSize().cy; break;
			case 1:
				xoffset = (GetDocument()->GetDocSize().cx + 7) >> 3;
				yoffset = (GetDocument()->GetDocSize().cy + 7) >> 3; break;
			case 2:
				xoffset = (GetDocument()->GetDocSize().cx + 3) >> 2;
				yoffset = (GetDocument()->GetDocSize().cy + 3) >> 2; break;
			case 3:
				xoffset = (GetDocument()->GetDocSize().cx * 3 + 7) >> 3;
				yoffset = (GetDocument()->GetDocSize().cy * 3 + 7) >> 3; break;
			case 4:
				xoffset = (GetDocument()->GetDocSize().cx + 1) >> 1;
				yoffset = (GetDocument()->GetDocSize().cy + 1) >> 1; break;
			case 5:
				xoffset = (GetDocument()->GetDocSize().cx * 5 + 7) >> 3;
				yoffset = (GetDocument()->GetDocSize().cy * 5 + 7) >> 3; break;
			case 6:
				xoffset = (GetDocument()->GetDocSize().cx * 3 + 3) >> 2;
				yoffset = (GetDocument()->GetDocSize().cy * 3 + 3) >> 2; break;
			case 7:
				xoffset = (GetDocument()->GetDocSize().cx * 7 + 7) >> 3;
				yoffset = (GetDocument()->GetDocSize().cy * 7 + 7) >> 3; break;
			}
			if (m_zoomval > 8)
			{
				xoffset += GetDocument()->GetDocSize().cx;
				yoffset += GetDocument()->GetDocSize().cy;
			}
			SetScrollSizes(MM_TEXT,	CSize(xoffset, yoffset));
		}
		if ((int)lHint >= 4 && (int)lHint <= 19) return;
	}
	if (lHint == 20) { SmartDither(); return; }
	m_left = 0;
	m_top = 0;
	m_right = 0;
	m_bottom = 0;
	if (((CDibLookApp *)AfxGetApp())->m_trim)
	{
		switch (GetDocument()->m_transform)
		{
		case 1:
		case 5:
			if (GetDocument()->m_image_width > GetDocument()->m_MCUwidth)
				m_right = GetDocument()->m_image_width % GetDocument()->m_MCUwidth;
			break;
		case 2:
		case 7:
			if (GetDocument()->m_image_height > GetDocument()->m_MCUheight)
				m_bottom = GetDocument()->m_image_height % GetDocument()->m_MCUheight;
			break;
		case 4:
		case 6:
			if (GetDocument()->m_image_width > GetDocument()->m_MCUwidth)
				m_right = GetDocument()->m_image_width % GetDocument()->m_MCUwidth;
			if (GetDocument()->m_image_height > GetDocument()->m_MCUheight)
				m_bottom = GetDocument()->m_image_height % GetDocument()->m_MCUheight;
			break;
		}
	}
	if (lHint == 3) return;
	char buf[64];
	unsigned int paru[2];
	int pari[2];
	sprintf(buf, " %dx%d+%d+%d",
		GetDocument()->m_image_width - m_right,
		GetDocument()->m_image_height - m_bottom, 0, 0);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);
}

void CDibView::OnActivateView(BOOL bActivate, CView* pActivateView,
					CView* pDeactiveView)
{
	CScrollView::OnActivateView(bActivate, pActivateView, pDeactiveView);

	if (bActivate)
	{
		ASSERT(pActivateView == this);
		OnDoRealize((WPARAM)m_hWnd, 0);   // same as SendMessage(WM_DOREALIZE);

		char buf[64];
		unsigned int paru[2];
		int pari[2];
		sprintf(buf, " %dx%d+%d+%d",
			GetDocument()->m_image_width - m_right - m_left,
			GetDocument()->m_image_height - m_bottom - m_top, m_left, m_top);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
		((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	}
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, NULL);
}

void CDibView::OnEditCopy()
{
	CDibDoc* pDoc = GetDocument();
	// Clean clipboard of contents, and copy the DIB.

	if (OpenClipboard())
	{
		BeginWaitCursor();
		EmptyClipboard();
		SetClipboardData (CF_DIB, CopyHandle((HANDLE) pDoc->GetHDIB()) );
		CloseClipboard();
		EndWaitCursor();
	}
}



void CDibView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(GetDocument()->GetHDIB() != NULL);
}


void CDibView::OnEditPaste()
{
	HDIB hNewDIB = NULL;

	if (OpenClipboard())
	{
		BeginWaitCursor();

		hNewDIB = (HDIB) CopyHandle(::GetClipboardData(CF_DIB));

		CloseClipboard();

		if (hNewDIB != NULL)
		{
			CDibDoc* pDoc = GetDocument();
			pDoc->ReplaceHDIB(hNewDIB); // and free the old DIB
			pDoc->InitDIBData();    // set up new size & palette
			pDoc->SetModifiedFlag(TRUE);

			SetScrollSizes(MM_TEXT, pDoc->GetDocSize());
			OnDoRealize((WPARAM)m_hWnd,0);  // realize the new palette
			pDoc->UpdateAllViews(NULL);
		}
		EndWaitCursor();
	}
}


void CDibView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(::IsClipboardFormatAvailable(CF_DIB));
}


void CDibView::OnEditSelectAll()
{
	OnUpdate(NULL, 1);
}


void CDibView::OnIndicatorPos()
{
}


void CDibView::OnUpdateIndicatorPos(CCmdUI* pCmdUI)
{
}


void CDibView::OnLButtonDown(UINT nFlags, CPoint point)
{
	CScrollView::OnLButtonDown(nFlags, point);

	if (m_flag == 0)
	{
		m_x = point.x;
		m_y = point.y;
		m_flag = 1;
	}
}


void CDibView::OnLButtonUp(UINT nFlags, CPoint point)
{
	CScrollView::OnLButtonUp(nFlags, point);

    TRACE("OnLButtonUp nFlags %d, m_flag %d\n", nFlags, m_flag);
	if (m_flag)
	{
		if ((m_flag == 2 || m_flag == 101) &&
			((CDibLookApp *)AfxGetApp())->m_show_cropmask == 0)
			OnUpdate(NULL);
		SetCursor(LoadCursor(NULL, IDC_ARROW));
		m_flag = 0;
        TRACE("OnLButtonUp SetCursor ARROW m_flag %d\n", m_flag);
	}
}


void CDibView::OnRButtonDown(UINT nFlags, CPoint point)
{
	CScrollView::OnRButtonDown(nFlags, point);

    TRACE("OnRButtonDown nFlags %d, m_flag %d\n", nFlags, m_flag);
	CMenu mnuTop;
	mnuTop.LoadMenu(IDR_POPUP1);
	CMenu* pPopup = mnuTop.GetSubMenu(0);
	ClientToScreen(&point);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON,
		point.x, point.y, this);
}


void CDibView::OnMouseMove(UINT nFlags, CPoint point)
{
  CScrollView::OnMouseMove(nFlags, point);

  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  CDibDoc *pDoc = GetDocument();

  CPoint scroll = GetDeviceScrollPosition();
  int x, y, scale;
  scale = pDoc->m_scale;
  if (scale == 8) scale = m_zoomval;
  switch (scale)
  {
  case 1:
	x = (point.x + scroll.x) << 3;
	y = (point.y + scroll.y) << 3; break;
  case 2:
	x = (point.x + scroll.x) << 2;
	y = (point.y + scroll.y) << 2; break;
  case 3:
	x = ((point.x + scroll.x) << 3) / 3;
	y = ((point.y + scroll.y) << 3) / 3; break;
  case 4:
	x = (point.x + scroll.x) << 1;
	y = (point.y + scroll.y) << 1; break;
  case 5:
	x = ((point.x + scroll.x) << 3) / 5;
	y = ((point.y + scroll.y) << 3) / 5; break;
  case 6:
	x = ((point.x + scroll.x) << 2) / 3;
	y = ((point.y + scroll.y) << 2) / 3; break;
  case 7:
	x = ((point.x + scroll.x) << 3) / 7;
	y = ((point.y + scroll.y) << 3) / 7; break;
  case 8:
	x = point.x + scroll.x;
	y = point.y + scroll.y; break;
  case 9:
	x = (((point.x + scroll.x) << 3) + 4) / 9;
	y = (((point.y + scroll.y) << 3) + 4) / 9; break;
  case 10:
	x = (((point.x + scroll.x) << 2) + 2) / 5;
	y = (((point.y + scroll.y) << 2) + 2) / 5; break;
  case 11:
	x = (((point.x + scroll.x) << 3) + 5) / 11;
	y = (((point.y + scroll.y) << 3) + 5) / 11; break;
  case 12:
	x = (((point.x + scroll.x) << 1) + 1) / 3;
	y = (((point.y + scroll.y) << 1) + 1) / 3; break;
  case 13:
	x = (((point.x + scroll.x) << 3) + 6) / 13;
	y = (((point.y + scroll.y) << 3) + 6) / 13; break;
  case 14:
	x = (((point.x + scroll.x) << 2) + 3) / 7;
	y = (((point.y + scroll.y) << 2) + 3) / 7; break;
  case 15:
	x = (((point.x + scroll.x) << 3) + 7) / 15;
	y = (((point.y + scroll.y) << 3) + 7) / 15; break;
  case 16:
	x = (point.x + scroll.x + 1) >> 1;
	y = (point.y + scroll.y + 1) >> 1; break;
  }
  char buf[64];
  unsigned int paru[2];
  int pari[2];

    TRACE("OnMouseMove: x %d, y %d, nFlags %d, m_flag %d\n", x, y, nFlags, m_flag);
  if (m_flag == 100)
  {
	int width = pDoc->m_image_width - m_right - m_left;
	int height = pDoc->m_image_height - m_bottom - m_top;
    m_move_offset_x = width / 2;
    m_move_offset_y = height / 2;
    TRACE("OnMouseMove: offset_x %d offset_y %d\n", m_move_offset_x, m_move_offset_y);
	m_left = x - m_move_offset_x; m_left -= m_left % pDoc->m_MCUwidth;
	if (m_left < 0) m_left = 0;
	m_top = y - m_move_offset_y; m_top -= m_top % pDoc->m_MCUheight;
	if (m_top < 0) m_top = 0;
	m_right = pDoc->m_image_width - m_left - width;
	m_bottom = pDoc->m_image_height - m_top - height;
	int lim_x = 0;
	int lim_y = 0;
	if (myapp->m_trim)
	{
		switch (pDoc->m_transform)
		{
		case 1:
		case 5:
			if (width > pDoc->m_image_width % pDoc->m_MCUwidth)
				lim_x = pDoc->m_image_width % pDoc->m_MCUwidth;
			break;
		case 2:
		case 7:
			if (height > pDoc->m_image_height % pDoc->m_MCUheight)
				lim_y = pDoc->m_image_height % pDoc->m_MCUheight;
			break;
		case 4:
		case 6:
			if (width > pDoc->m_image_width % pDoc->m_MCUwidth)
				lim_x = pDoc->m_image_width % pDoc->m_MCUwidth;
			if (height > pDoc->m_image_height % pDoc->m_MCUheight)
				lim_y = pDoc->m_image_height % pDoc->m_MCUheight;
			break;
		}
	}
	if (m_right < lim_x)
	{
	  m_left += m_right - lim_x;
	  m_right = lim_x + (m_left % pDoc->m_MCUwidth);
	  m_left -= m_right - lim_x;
	}
	if (m_bottom < lim_y)
	{
	  m_top += m_bottom - lim_y;
	  m_bottom = lim_y + (m_top % pDoc->m_MCUheight);
	  m_top -= m_bottom - lim_y;
	}
	if (myapp->m_show_cropmask) OnUpdate(NULL);
	else
	{
    TRACE("OnPopupMoveFrame: WHAT IS HERE, m_flag set to %d\n", m_flag);
		SIZE size;
		RECT rect2;
		size.cx = 1;
		size.cy = 1;
		switch (scale & 7)
		{
		case 0:
			rect2.left = m_left;
			rect2.top = m_top;
			rect2.right = pDoc->m_image_width - m_right;
			rect2.bottom = pDoc->m_image_height - m_bottom; break;
		case 1:
			rect2.left = (m_left + 7) >> 3;
			rect2.top = (m_top + 7) >> 3;
			rect2.right = (pDoc->m_image_width - m_right + 7) >> 3;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
		case 2:
			rect2.left = (m_left + 3) >> 2;
			rect2.top = (m_top + 3) >> 2;
			rect2.right = (pDoc->m_image_width - m_right + 3) >> 2;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
		case 3:
			rect2.left = (m_left * 3 + 7) >> 3;
			rect2.top = (m_top * 3 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
		case 4:
			rect2.left = (m_left + 1) >> 1;
			rect2.top = (m_top + 1) >> 1;
			rect2.right = (pDoc->m_image_width - m_right + 1) >> 1;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
		case 5:
			rect2.left = (m_left * 5 + 7) >> 3;
			rect2.top = (m_top * 5 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
		case 6:
			rect2.left = (m_left * 3 + 3) >> 2;
			rect2.top = (m_top * 3 + 3) >> 2;
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
		case 7:
			rect2.left = (m_left * 7 + 7) >> 3;
			rect2.top = (m_top * 7 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
		}
		if (scale > 8)
		{
			rect2.left += m_left;
			rect2.top += m_top;
			rect2.right += pDoc->m_image_width - m_right;
			rect2.bottom += pDoc->m_image_height - m_bottom;
		}
		rect2.left -= scroll.x;
		rect2.top -= scroll.y;
		rect2.right -= scroll.x;
		rect2.bottom -= scroll.y;
		GetDC()->DrawDragRect(&rect2, size, NULL, size);
	}
    SetCursor(LoadCursor(NULL, IDC_SIZEALL));
	m_flag = 101;
    TRACE("OnPopupMoveFrame: SetCursor SIZEALL, m_flag set to %d\n", m_flag);
	return;
  }
  if (m_flag == 101)
  {
	SIZE size;
	RECT rect1, rect2;
	size.cx = 1;
	size.cy = 1;
	switch (scale & 7)
	{
	case 0:
		rect1.left = m_left;
		rect1.top = m_top;
		rect1.right = pDoc->m_image_width - m_right;
		rect1.bottom = pDoc->m_image_height - m_bottom; break;
	case 1:
		rect1.left = (m_left + 7) >> 3;
		rect1.top = (m_top + 7) >> 3;
		rect1.right = (pDoc->m_image_width - m_right + 7) >> 3;
		rect1.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
	case 2:
		rect1.left = (m_left + 3) >> 2;
		rect1.top = (m_top + 3) >> 2;
		rect1.right = (pDoc->m_image_width - m_right + 3) >> 2;
		rect1.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
	case 3:
		rect1.left = (m_left * 3 + 7) >> 3;
		rect1.top = (m_top * 3 + 7) >> 3;
		rect1.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
		rect1.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
	case 4:
		rect1.left = (m_left + 1) >> 1;
		rect1.top = (m_top + 1) >> 1;
		rect1.right = (pDoc->m_image_width - m_right + 1) >> 1;
		rect1.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
	case 5:
		rect1.left = (m_left * 5 + 7) >> 3;
		rect1.top = (m_top * 5 + 7) >> 3;
		rect1.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
		rect1.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
	case 6:
		rect1.left = (m_left * 3 + 3) >> 2;
		rect1.top = (m_top * 3 + 3) >> 2;
		rect1.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
		rect1.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
	case 7:
		rect1.left = (m_left * 7 + 7) >> 3;
		rect1.top = (m_top * 7 + 7) >> 3;
		rect1.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
		rect1.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
	}
	if (scale > 8)
	{
		rect1.left += m_left;
		rect1.top += m_top;
		rect1.right += pDoc->m_image_width - m_right;
		rect1.bottom += pDoc->m_image_height - m_bottom;
	}
	rect1.left -= scroll.x;
	rect1.top -= scroll.y;
	rect1.right -= scroll.x;
	rect1.bottom -= scroll.y;
	int width = pDoc->m_image_width - m_right - m_left;
	int height = pDoc->m_image_height - m_bottom - m_top;
	m_left = x - m_move_offset_x; m_left -= m_left % pDoc->m_MCUwidth;
	if (m_left < 0) m_left = 0;
	m_top = y - m_move_offset_y; m_top -= m_top % pDoc->m_MCUheight;
	if (m_top < 0) m_top = 0;
	m_right = pDoc->m_image_width - m_left - width;
	m_bottom = pDoc->m_image_height - m_top - height;
	int lim_x = 0;
	int lim_y = 0;
	if (myapp->m_trim)
	{
		switch (pDoc->m_transform)
		{
		case 1:
		case 5:
			if (width > pDoc->m_image_width % pDoc->m_MCUwidth)
				lim_x = pDoc->m_image_width % pDoc->m_MCUwidth;
			break;
		case 2:
		case 7:
			if (height > pDoc->m_image_height % pDoc->m_MCUheight)
				lim_y = pDoc->m_image_height % pDoc->m_MCUheight;
			break;
		case 4:
		case 6:
			if (width > pDoc->m_image_width % pDoc->m_MCUwidth)
				lim_x = pDoc->m_image_width % pDoc->m_MCUwidth;
			if (height > pDoc->m_image_height % pDoc->m_MCUheight)
				lim_y = pDoc->m_image_height % pDoc->m_MCUheight;
			break;
		}
	}
	if (m_right < lim_x)
	{
	  m_left += m_right - lim_x;
	  m_right = lim_x + (m_left % pDoc->m_MCUwidth);
	  m_left -= m_right - lim_x;
	}
	if (m_bottom < lim_y)
	{
	  m_top += m_bottom - lim_y;
	  m_bottom = lim_y + (m_top % pDoc->m_MCUheight);
	  m_top -= m_bottom - lim_y;
	}
	if (myapp->m_show_cropmask) OnUpdate(NULL);
	else
	{
		switch (scale & 7)
		{
		case 0:
			rect2.left = m_left;
			rect2.top = m_top;
			rect2.right = pDoc->m_image_width - m_right;
			rect2.bottom = pDoc->m_image_height - m_bottom; break;
		case 1:
			rect2.left = (m_left + 7) >> 3;
			rect2.top = (m_top + 7) >> 3;
			rect2.right = (pDoc->m_image_width - m_right + 7) >> 3;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
		case 2:
			rect2.left = (m_left + 3) >> 2;
			rect2.top = (m_top + 3) >> 2;
			rect2.right = (pDoc->m_image_width - m_right + 3) >> 2;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
		case 3:
			rect2.left = (m_left * 3 + 7) >> 3;
			rect2.top = (m_top * 3 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
		case 4:
			rect2.left = (m_left + 1) >> 1;
			rect2.top = (m_top + 1) >> 1;
			rect2.right = (pDoc->m_image_width - m_right + 1) >> 1;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
		case 5:
			rect2.left = (m_left * 5 + 7) >> 3;
			rect2.top = (m_top * 5 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
		case 6:
			rect2.left = (m_left * 3 + 3) >> 2;
			rect2.top = (m_top * 3 + 3) >> 2;
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
		case 7:
			rect2.left = (m_left * 7 + 7) >> 3;
			rect2.top = (m_top * 7 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
		}
		if (scale > 8)
		{
			rect2.left += m_left;
			rect2.top += m_top;
			rect2.right += pDoc->m_image_width - m_right;
			rect2.bottom += pDoc->m_image_height - m_bottom;
		}
		rect2.left -= scroll.x;
		rect2.top -= scroll.y;
		rect2.right -= scroll.x;
		rect2.bottom -= scroll.y;
		GetDC()->DrawDragRect(&rect2, size, &rect1, size);
	}
    SetCursor(LoadCursor(NULL, IDC_SIZEALL));

//	CRect rcDisp;
//	GetClientRect(rcDisp);
//	if (point.x < rcDisp.left)
//		ScrollToPosition(CPoint(point.x, scroll.y));

	sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
    TRACE("OnPopupMoveFrame: SetCursor SIZEALL, m_flag set to %d (print status line)\n", m_flag);
	return;
  }

  if (nFlags & MK_LBUTTON)
  {
	switch (m_flag) {
	case 1:
	  if (point.x == m_x && point.y == m_y) break;
	  switch (scale)
	  {
	  case 1:
		m_left = (m_x + scroll.x) << 3;
		m_top = (m_y + scroll.y) << 3; break;
	  case 2:
		m_left = (m_x + scroll.x) << 2;
		m_top = (m_y + scroll.y) << 2; break;
	  case 3:
		m_left = ((m_x + scroll.x) << 3) / 3;
		m_top = ((m_y + scroll.y) << 3) / 3; break;
	  case 4:
		m_left = (m_x + scroll.x) << 1;
		m_top = (m_y + scroll.y) << 1; break;
	  case 5:
		m_left = ((m_x + scroll.x) << 3) / 5;
		m_top = ((m_y + scroll.y) << 3) / 5; break;
	  case 6:
		m_left = ((m_x + scroll.x) << 2) / 3;
		m_top = ((m_y + scroll.y) << 2) / 3; break;
	  case 7:
		m_left = ((m_x + scroll.x) << 3) / 7;
		m_top = ((m_y + scroll.y) << 3) / 7; break;
	  case 8:
		m_left = m_x + scroll.x;
		m_top = m_y + scroll.y; break;
	  case 9:
		m_left = (((m_x + scroll.x) << 3) + 4) / 9;
		m_top = (((m_y + scroll.y) << 3) + 4) / 9; break;
	  case 10:
		m_left = (((m_x + scroll.x) << 2) + 2) / 5;
		m_top = (((m_y + scroll.y) << 2) + 2) / 5; break;
	  case 11:
		m_left = (((m_x + scroll.x) << 3) + 5) / 11;
		m_top = (((m_y + scroll.y) << 3) + 5) / 11; break;
	  case 12:
		m_left = (((m_x + scroll.x) << 1) + 1) / 3;
		m_top = (((m_y + scroll.y) << 1) + 1) / 3; break;
	  case 13:
		m_left = (((m_x + scroll.x) << 3) + 6) / 13;
		m_top = (((m_y + scroll.y) << 3) + 6) / 13; break;
	  case 14:
		m_left = (((m_x + scroll.x) << 2) + 3) / 7;
		m_top = (((m_y + scroll.y) << 2) + 3) / 7; break;
	  case 15:
		m_left = (((m_x + scroll.x) << 3) + 7) / 15;
		m_top = (((m_y + scroll.y) << 3) + 7) / 15; break;
	  case 16:
		m_left = (m_x + scroll.x + 1) >> 1;
		m_top = (m_y + scroll.y + 1) >> 1; break;
	  }
	  if (m_left >= pDoc->m_image_width) m_left = pDoc->m_image_width - 1;
	  if (m_top >= pDoc->m_image_height) m_top = pDoc->m_image_height - 1;
	  m_left -= m_left % pDoc->m_MCUwidth;
	  m_top -= m_top % pDoc->m_MCUheight;
	  if (x < m_left) x = m_left;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (x >= pDoc->m_image_width) x = pDoc->m_image_width - 1;
	  if (y >= pDoc->m_image_height) y = pDoc->m_image_height - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 1:
		case 5:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			break;
		case 2:
		case 7:
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		case 4:
		case 6:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		}
	  }
	  m_right = pDoc->m_image_width - 1 - x;
	  m_bottom = pDoc->m_image_height - 1 - y;
	  if (myapp->m_show_cropmask) OnUpdate(NULL);
	  else
	  {
		SIZE size;
		RECT rect2;
		size.cx = 1;
		size.cy = 1;
		switch (scale & 7)
		{
		case 0:
			rect2.left = m_left;
			rect2.top = m_top;
			rect2.right = pDoc->m_image_width - m_right;
			rect2.bottom = pDoc->m_image_height - m_bottom; break;
		case 1:
			rect2.left = (m_left + 7) >> 3;
			rect2.top = (m_top + 7) >> 3;
			rect2.right = (pDoc->m_image_width - m_right + 7) >> 3;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
		case 2:
			rect2.left = (m_left + 3) >> 2;
			rect2.top = (m_top + 3) >> 2;
			rect2.right = (pDoc->m_image_width - m_right + 3) >> 2;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
		case 3:
			rect2.left = (m_left * 3 + 7) >> 3;
			rect2.top = (m_top * 3 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
		case 4:
			rect2.left = (m_left + 1) >> 1;
			rect2.top = (m_top + 1) >> 1;
			rect2.right = (pDoc->m_image_width - m_right + 1) >> 1;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
		case 5:
			rect2.left = (m_left * 5 + 7) >> 3;
			rect2.top = (m_top * 5 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
		case 6:
			rect2.left = (m_left * 3 + 3) >> 2;
			rect2.top = (m_top * 3 + 3) >> 2;
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
		case 7:
			rect2.left = (m_left * 7 + 7) >> 3;
			rect2.top = (m_top * 7 + 7) >> 3;
			rect2.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
		}
		if (scale > 8)
		{
			rect2.left += m_left;
			rect2.top += m_top;
			rect2.right += pDoc->m_image_width - m_right;
			rect2.bottom += pDoc->m_image_height - m_bottom;
		}
		rect2.left -= scroll.x;
		rect2.top -= scroll.y;
		rect2.right -= scroll.x;
		rect2.bottom -= scroll.y;
		GetDC()->DrawDragRect(&rect2, size, NULL, size);
	  }
	  m_flag = 2;
	  break;
	case 2:
	  if (x < m_left) x = m_left;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (x >= pDoc->m_image_width) x = pDoc->m_image_width - 1;
	  if (y >= pDoc->m_image_height) y = pDoc->m_image_height - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 1:
		case 5:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			break;
		case 2:
		case 7:
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		case 4:
		case 6:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		}
	  }
	  if (myapp->m_show_cropmask)
	  {
		m_right = pDoc->m_image_width - 1 - x;
		m_bottom = pDoc->m_image_height - 1 - y;
		OnUpdate(NULL);
	  }
	  else
	  {
		SIZE size;
		RECT rect1, rect2;
		size.cx = 1;
		size.cy = 1;
		switch (scale & 7)
		{
		case 0:
			rect1.left = m_left;
			rect1.top = m_top;
			rect1.right = pDoc->m_image_width - m_right;
			rect1.bottom = pDoc->m_image_height - m_bottom; break;
		case 1:
			rect1.left = (m_left + 7) >> 3;
			rect1.top = (m_top + 7) >> 3;
			rect1.right = (pDoc->m_image_width - m_right + 7) >> 3;
			rect1.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
		case 2:
			rect1.left = (m_left + 3) >> 2;
			rect1.top = (m_top + 3) >> 2;
			rect1.right = (pDoc->m_image_width - m_right + 3) >> 2;
			rect1.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
		case 3:
			rect1.left = (m_left * 3 + 7) >> 3;
			rect1.top = (m_top * 3 + 7) >> 3;
			rect1.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
			rect1.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
		case 4:
			rect1.left = (m_left + 1) >> 1;
			rect1.top = (m_top + 1) >> 1;
			rect1.right = (pDoc->m_image_width - m_right + 1) >> 1;
			rect1.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
		case 5:
			rect1.left = (m_left * 5 + 7) >> 3;
			rect1.top = (m_top * 5 + 7) >> 3;
			rect1.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
			rect1.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
		case 6:
			rect1.left = (m_left * 3 + 3) >> 2;
			rect1.top = (m_top * 3 + 3) >> 2;
			rect1.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
			rect1.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
		case 7:
			rect1.left = (m_left * 7 + 7) >> 3;
			rect1.top = (m_top * 7 + 7) >> 3;
			rect1.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
			rect1.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
		}
		if (scale > 8)
		{
			rect1.left += m_left;
			rect1.top += m_top;
			rect1.right += pDoc->m_image_width - m_right;
			rect1.bottom += pDoc->m_image_height - m_bottom;
		}
		rect1.left -= scroll.x;
		rect1.top -= scroll.y;
		rect1.right -= scroll.x;
		rect1.bottom -= scroll.y;
		rect2.left = rect1.left;
		rect2.top = rect1.top;
		m_right = pDoc->m_image_width - 1 - x;
		m_bottom = pDoc->m_image_height - 1 - y;
		switch (scale & 7)
		{
		case 0:
			rect2.right = pDoc->m_image_width - m_right;
			rect2.bottom = pDoc->m_image_height - m_bottom; break;
		case 1:
			rect2.right = (pDoc->m_image_width - m_right + 7) >> 3;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 7) >> 3; break;
		case 2:
			rect2.right = (pDoc->m_image_width - m_right + 3) >> 2;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 3) >> 2; break;
		case 3:
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
		case 4:
			rect2.right = (pDoc->m_image_width - m_right + 1) >> 1;
			rect2.bottom = (pDoc->m_image_height - m_bottom + 1) >> 1; break;
		case 5:
			rect2.right = ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
		case 6:
			rect2.right = ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
		case 7:
			rect2.right = ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
			rect2.bottom = ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
		}
		if (scale > 8)
		{
			rect2.right += pDoc->m_image_width - m_right;
			rect2.bottom += pDoc->m_image_height - m_bottom;
		}
		rect2.right -= scroll.x;
		rect2.bottom -= scroll.y;
		GetDC()->DrawDragRect(&rect2, size, &rect1, size);
	  }

	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  break;
	case 4:
	  if (x > pDoc->m_image_width - 1 - m_right) x = pDoc->m_image_width - 1 - m_right;
	  x -= x % pDoc->m_MCUwidth;
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  if (x == m_left) break;
	  m_left = x;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZEWE, m_flag %d\n", m_flag);
	  break;
	case 5:
	  if (y > pDoc->m_image_height - 1 - m_bottom) y = pDoc->m_image_height - 1 - m_bottom;
	  y -= y % pDoc->m_MCUheight;
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  if (y == m_top) break;
	  m_top = y;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZENS, m_flag %d\n", m_flag);
	  break;
	case 6:
	  if (x < m_left) x = m_left;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
	  }
	  if (x >= pDoc->m_image_width) x = pDoc->m_image_width - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 1:
		case 4:
		case 5:
		case 6:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			break;
		}
	  }
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  if (pDoc->m_image_width - 1 - x == m_right) break;
	  m_right = pDoc->m_image_width - 1 - x;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZEWE, m_flag %d\n", m_flag);
	  break;
	case 7:
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (y >= pDoc->m_image_height) y = pDoc->m_image_height - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 2:
		case 4:
		case 6:
		case 7:
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		}
	  }
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  if (pDoc->m_image_height - 1 - y == m_bottom) break;
	  m_bottom = pDoc->m_image_height - 1 - y;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZENS, m_flag %d\n", m_flag);
	  break;
	case 8:
	  if (x > pDoc->m_image_width - 1 - m_right) x = pDoc->m_image_width - 1 - m_right;
	  if (y > pDoc->m_image_height - 1 - m_bottom) y = pDoc->m_image_height - 1 - m_bottom;
	  x -= x % pDoc->m_MCUwidth;
	  y -= y % pDoc->m_MCUheight;
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  if (x == m_left && y == m_top) break;
	  m_left = x;
	  m_top = y;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZENWSE, m_flag %d\n", m_flag);
	  break;
	case 9:
	  if (x < m_left) x = m_left;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
	  }
	  if (x >= pDoc->m_image_width) x = pDoc->m_image_width - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 1:
		case 4:
		case 5:
		case 6:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			break;
		}
	  }
	  if (y > pDoc->m_image_height - 1 - m_bottom) y = pDoc->m_image_height - 1 - m_bottom;
	  y -= y % pDoc->m_MCUheight;
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  if (pDoc->m_image_width - 1 - x == m_right && y == m_top) break;
	  m_right = pDoc->m_image_width - 1 - x;
	  m_top = y;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZENESW, m_flag %d\n", m_flag);
	  break;
	case 10:
	  if (x < m_left) x = m_left;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		x -= x % pDoc->m_MCUwidth;
		x += pDoc->m_MCUwidth - 1;
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (x >= pDoc->m_image_width) x = pDoc->m_image_width - 1;
	  if (y >= pDoc->m_image_height) y = pDoc->m_image_height - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 1:
		case 5:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			break;
		case 2:
		case 7:
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		case 4:
		case 6:
			if (x >= pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) &&
				pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) > m_left)
			  x = pDoc->m_image_width - (pDoc->m_image_width % pDoc->m_MCUwidth) - 1;
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		}
	  }
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  if (pDoc->m_image_width - 1 - x == m_right && pDoc->m_image_height - 1 - y == m_bottom) break;
	  m_right = pDoc->m_image_width - 1 - x;
	  m_bottom = pDoc->m_image_height - 1 - y;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZENWSE, m_flag %d\n", m_flag);
	  break;
	case 11:
	  if (x > pDoc->m_image_width - 1 - m_right) x = pDoc->m_image_width - 1 - m_right;
	  x -= x % pDoc->m_MCUwidth;
	  if (y < m_top) y = m_top;
	  if (myapp->m_endpoint_snap)
	  {
		y -= y % pDoc->m_MCUheight;
		y += pDoc->m_MCUheight - 1;
	  }
	  if (y >= pDoc->m_image_height) y = pDoc->m_image_height - 1;
	  if (myapp->m_trim)
	  {
		switch (pDoc->m_transform)
		{
		case 2:
		case 4:
		case 6:
		case 7:
			if (y >= pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) &&
				pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) > m_top)
			  y = pDoc->m_image_height - (pDoc->m_image_height % pDoc->m_MCUheight) - 1;
			break;
		}
	  }
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  if (x == m_left && pDoc->m_image_height - 1 - y == m_bottom) break;
	  m_left = x;
	  m_bottom = pDoc->m_image_height - 1 - y;
	  sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	  ((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);
	  OnUpdate(NULL);
      TRACE("OnPopupMoveFrame: SetCursor SIZENESW, m_flag %d\n", m_flag);
	  break;
    case 99:
        m_flag = 101;
        TRACE("OnPopupMoveFrame: turn on move, m_flag %d\n", m_flag);
        break;
	}
  }
  else
  {
	int x1, y1, x2, y2;
	x1 = point.x + scroll.x;
	y1 = point.y + scroll.y;
	x2 = x1 + 1;
	y2 = y1 + 1;
	switch (scale & 7)
	{
	case 0:
		x1 -= m_left;
		y1 -= m_top;
		x2 -= pDoc->m_image_width - m_right;
		y2 -= pDoc->m_image_height - m_bottom; break;
	case 1:
		x1 -= (m_left + 7) >> 3;
		y1 -= (m_top + 7) >> 3;
		x2 -= (pDoc->m_image_width - m_right + 7) >> 3;
		y2 -= (pDoc->m_image_height - m_bottom + 7) >> 3; break;
	case 2:
		x1 -= (m_left + 3) >> 2;
		y1 -= (m_top + 3) >> 2;
		x2 -= (pDoc->m_image_width - m_right + 3) >> 2;
		y2 -= (pDoc->m_image_height - m_bottom + 3) >> 2; break;
	case 3:
		x1 -= (m_left * 3 + 7) >> 3;
		y1 -= (m_top * 3 + 7) >> 3;
		x2 -= ((pDoc->m_image_width - m_right) * 3 + 7) >> 3;
		y2 -= ((pDoc->m_image_height - m_bottom) * 3 + 7) >> 3; break;
	case 4:
		x1 -= (m_left + 1) >> 1;
		y1 -= (m_top + 1) >> 1;
		x2 -= (pDoc->m_image_width - m_right + 1) >> 1;
		y2 -= (pDoc->m_image_height - m_bottom + 1) >> 1; break;
	case 5:
		x1 -= (m_left * 5 + 7) >> 3;
		y1 -= (m_top * 5 + 7) >> 3;
		x2 -= ((pDoc->m_image_width - m_right) * 5 + 7) >> 3;
		y2 -= ((pDoc->m_image_height - m_bottom) * 5 + 7) >> 3; break;
	case 6:
		x1 -= (m_left * 3 + 3) >> 2;
		y1 -= (m_top * 3 + 3) >> 2;
		x2 -= ((pDoc->m_image_width - m_right) * 3 + 3) >> 2;
		y2 -= ((pDoc->m_image_height - m_bottom) * 3 + 3) >> 2; break;
	case 7:
		x1 -= (m_left * 7 + 7) >> 3;
		y1 -= (m_top * 7 + 7) >> 3;
		x2 -= ((pDoc->m_image_width - m_right) * 7 + 7) >> 3;
		y2 -= ((pDoc->m_image_height - m_bottom) * 7 + 7) >> 3; break;
	}
	if (scale > 8)
	{
		x1 -= m_left;
		y1 -= m_top;
		x2 -= pDoc->m_image_width - m_right;
		y2 -= pDoc->m_image_height - m_bottom;
	}
    int ox1, ox2, oy1, oy2;
    ox1 = x1;
    ox2 = x2;
    oy1 = y1;
    oy2 = y2;
	if (x1 < 0) x1 = -x1;
	if (x2 < 0) x2 = -x2;
	if (y1 < 0) y1 = -y1;
	if (y2 < 0) y2 = -y2;
    TRACE("OnPopupMoveFrame: no button pressed, m_flag %d, x1 %d, x2 %d, y1 %d, y2 %d\n", m_flag, x1, x2, y1, y2);

	if (x1 <= 1 && y1 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  m_flag = 8;
      TRACE("OnPopupMoveFrame: SetCursor SIZENWSE, m_flag %d\n", m_flag);
	}
	else if (x2 <= 1 && y1 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  m_flag = 9;
      TRACE("OnPopupMoveFrame: SetCursor SIZENESW, m_flag %d\n", m_flag);
	}
	else if (x2 <= 1 && y2 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENWSE));
	  m_flag = 10;
      TRACE("OnPopupMoveFrame: SetCursor SIZENWSE, m_flag %d\n", m_flag);
	}
	else if (x1 <= 1 && y2 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENESW));
	  m_flag = 11;
      TRACE("OnPopupMoveFrame: SetCursor SIZENESW, m_flag %d\n", m_flag);
	}
	else if (x1 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  m_flag = 4;
      TRACE("OnPopupMoveFrame: SetCursor SIZEWE, m_flag %d\n", m_flag);
	}
	else if (y1 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  m_flag = 5;
      TRACE("OnPopupMoveFrame: SetCursor SIZENS, m_flag %d\n", m_flag);
	}
	else if (x2 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZEWE));
	  m_flag = 6;
      TRACE("OnPopupMoveFrame: SetCursor SIZEWE, m_flag %d\n", m_flag);
	}
	else if (y2 <= 1)
	{
	  SetCursor(LoadCursor(NULL, IDC_SIZENS));
	  m_flag = 7;
      TRACE("OnPopupMoveFrame: SetCursor SIZENS, m_flag %d\n", m_flag);
	}
	else if ( (m_left || m_top || m_right || m_bottom) &&
              (ox1 > 1 && ox2 < 1 && oy1 > 1 && oy2 < 1) )
	{
      SetCursor(LoadCursor(NULL, IDC_SIZEALL));
	  m_flag = 99;

      int offset_x = ox1;
      int offset_y = oy1;

	    switch (scale)
	    {
	    case 0:
            break;
	    case 1:
            offset_x *= 8;
            offset_y *= 8;
            break;
	    case 2:
            offset_x *= 4;
            offset_y *= 4;
            break;
	    case 3:
            offset_x *= 8; offset_x /= 3;
            offset_y *= 8; offset_y /= 3;
            break;
	    case 4:
            offset_x *= 2;
            offset_y *= 2;
            break;
	    case 5:
            offset_x *= 8; offset_x /= 5;
            offset_y *= 8; offset_y /= 5;
            break;
	    case 6:
            offset_x *= 8; offset_x /= 6;
            offset_y *= 8; offset_y /= 6;
            break;
	    case 7:
            offset_x *= 8; offset_x /= 7;
            offset_y *= 8; offset_y /= 7;
            break;
	    case 8:
            break;
	    case 9:
            offset_x *= 8; offset_x /= 9;
            offset_y *= 8; offset_y /= 9;
            break;
	    case 10:
            offset_x *= 8; offset_x /= 10;
            offset_y *= 8; offset_y /= 10;
            break;
	    case 11:
            offset_x *= 8; offset_x /= 11;
            offset_y *= 8; offset_y /= 11;
            break;
	    case 12:
            offset_x *= 8; offset_x /= 12;
            offset_y *= 8; offset_y /= 12;
            break;
	    case 13:
            offset_x *= 8; offset_x /= 13;
            offset_y *= 8; offset_y /= 13;
            break;
	    case 14:
            offset_x *= 8; offset_x /= 14;
            offset_y *= 8; offset_y /= 14;
            break;
	    case 15:
            offset_x *= 8; offset_x /= 15;
            offset_y *= 8; offset_y /= 15;
            break;
	    case 16:
            offset_x /= 2;
            offset_y /= 2;
            break;
	    }

      m_move_offset_x = offset_x;
      m_move_offset_y = offset_y;
      TRACE("OnPopupMoveFrame: SetCursor SIZEALL, ox1 %d, oy1 %d, scale %d, m_flag %d\n", ox1, oy1, scale, m_flag);
	}
	else if (m_flag && m_flag != 99)
	{
	  SetCursor(LoadCursor(NULL, IDC_ARROW));
	  m_flag = 0;
      TRACE("OnPopupMoveFrame: SetCursor ARROW, m_flag %d\n", m_flag);
	}
  }
}


void CDibView::OnPopup1MoveFrame()
{
  TRACE("OnPopupMoveFrame: m_flag %d\n", m_flag);
  SetCursor(LoadCursor(NULL, IDC_SIZEALL));
  m_flag = 100;
  TRACE("OnPopupMoveFrame: SetCursor SIZEALL, m_flag set to %d\n", m_flag);
}


static jpeg_transform_info cropinfo;


class CFrameDlg : public CDialog
{
public:
	CFrameDlg() : CDialog(CFrameDlg::IDD)
		{
			//{{AFX_DATA_INIT(CFrameDlg)
			//}}AFX_DATA_INIT
		}
// Dialog Data
	//{{AFX_DATA(CFrameDlg)
		enum { IDD = IDD_FRAMEBOX };
	//}}AFX_DATA
	int m_width;
	int m_height;
	int m_xoffset;
	int m_yoffset;
// Implementation
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	virtual void OnCheck1();
	virtual void OnRadio1();
	virtual void OnRadio2();
	//{{AFX_MSG(CFrameDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

void CFrameDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrefDlg)
	//}}AFX_DATA_MAP
}

BOOL CFrameDlg::OnInitDialog()
{
	char buffer[128];
	int width;
	int height;
	int xoffset;
	int yoffset;
	if (((CDibLookApp *)AfxGetApp())->m_actual)
	{
		((CButton *)GetDlgItem(IDC_RADIO2))->SetCheck(TRUE);
		width = m_width;
		height = m_height;
		xoffset = m_xoffset;
		yoffset = m_yoffset;
	}
	else
	{
		((CButton *)GetDlgItem(IDC_RADIO1))->SetCheck(TRUE);
		width = ((CDibLookApp *)AfxGetApp())->m_width;
		height = ((CDibLookApp *)AfxGetApp())->m_height;
		xoffset = ((CDibLookApp *)AfxGetApp())->m_xoffset;
		yoffset = ((CDibLookApp *)AfxGetApp())->m_yoffset;
	}
	sprintf(buffer, "%d", width);
	SetDlgItemText(IDC_EDIT1, buffer);
	sprintf(buffer, "%d", height);
	SetDlgItemText(IDC_EDIT2, buffer);
	if (xoffset < 0)
		buffer[0] = 0;
	else
		sprintf(buffer, "%d", xoffset);
	SetDlgItemText(IDC_EDIT3, buffer);
	if (yoffset < 0)
		buffer[0] = 0;
	else
		sprintf(buffer, "%d", yoffset);
	SetDlgItemText(IDC_EDIT4, buffer);
	if (xoffset < 0)
		sprintf(buffer, "%dx%d", width, height);
	else if (yoffset < 0)
		sprintf(buffer, "%dx%d+%d", width, height, xoffset);
	else
		sprintf(buffer, "%dx%d+%d+%d", width, height, xoffset, yoffset);
	SetDlgItemText(IDC_EDIT5, buffer);
	if (((CDibLookApp *)AfxGetApp())->m_cropspec)
	{
		((CButton *)GetDlgItem(IDC_CHECK1))->SetCheck(TRUE);
		GetDlgItem(IDC_EDIT1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT3)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT4)->EnableWindow(FALSE);
	}
	else
		GetDlgItem(IDC_EDIT5)->EnableWindow(FALSE);
	CDialog::OnInitDialog();
	return TRUE;
}

void CFrameDlg::OnOK()
{
	CDialog::OnOK();
	char buffer[128];
	if (((CButton *)GetDlgItem(IDC_RADIO1))->GetCheck())
		((CDibLookApp *)AfxGetApp())->m_actual = 0;
	else
		((CDibLookApp *)AfxGetApp())->m_actual = 1;
	if (((CButton *)GetDlgItem(IDC_CHECK1))->GetCheck())
	{
		GetDlgItemText(IDC_EDIT5, buffer, sizeof(buffer));
		if (jtransform_parse_crop_spec(&cropinfo, buffer))
		{
			if (cropinfo.crop_width_set == JCROP_POS)
			{
				sprintf(buffer, "%d", cropinfo.crop_width);
				SetDlgItemText(IDC_EDIT1, buffer);
			}
			if (cropinfo.crop_height_set == JCROP_POS)
			{
				sprintf(buffer, "%d", cropinfo.crop_height);
				SetDlgItemText(IDC_EDIT2, buffer);
			}
			if (cropinfo.crop_xoffset_set == JCROP_POS)
			{
				sprintf(buffer, "%d", cropinfo.crop_xoffset);
				SetDlgItemText(IDC_EDIT3, buffer);
			}
			if (cropinfo.crop_yoffset_set == JCROP_POS)
			{
				sprintf(buffer, "%d", cropinfo.crop_yoffset);
				SetDlgItemText(IDC_EDIT4, buffer);
			}
		}
		((CDibLookApp *)AfxGetApp())->m_cropspec = 1;
	}
	else
		((CDibLookApp *)AfxGetApp())->m_cropspec = 0;
	GetDlgItemText(IDC_EDIT1, buffer, sizeof(buffer));
	((CDibLookApp *)AfxGetApp())->m_width = atoi(buffer);
	GetDlgItemText(IDC_EDIT2, buffer, sizeof(buffer));
	((CDibLookApp *)AfxGetApp())->m_height = atoi(buffer);
	GetDlgItemText(IDC_EDIT3, buffer, sizeof(buffer));
	if (buffer[0])
		((CDibLookApp *)AfxGetApp())->m_xoffset = atoi(buffer);
	GetDlgItemText(IDC_EDIT4, buffer, sizeof(buffer));
	if (buffer[0])
		((CDibLookApp *)AfxGetApp())->m_yoffset = atoi(buffer);
}

void CFrameDlg::OnCheck1()
{
	char buffer1[32];
	char buffer2[32];
	char buffer3[32];
	char buffer4[32];
	char buf[64];

	if (((CButton *)GetDlgItem(IDC_CHECK1))->GetCheck())
	{
		GetDlgItemText(IDC_EDIT1, buffer1, sizeof(buffer1));
		GetDlgItemText(IDC_EDIT2, buffer2, sizeof(buffer2));
		GetDlgItemText(IDC_EDIT3, buffer3, sizeof(buffer3));
		GetDlgItemText(IDC_EDIT4, buffer4, sizeof(buffer4));
		if (buffer3[0] == 0)
			sprintf(buf, "%sx%s", buffer1, buffer2);
		else if (buffer4[0] == 0)
			sprintf(buf, "%sx%s+%s", buffer1, buffer2, buffer3);
		else
			sprintf(buf, "%sx%s+%s+%s", buffer1, buffer2, buffer3, buffer4);
		SetDlgItemText(IDC_EDIT5, buf);
		GetDlgItem(IDC_EDIT1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT3)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT4)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT5)->EnableWindow(TRUE);
	}
	else
	{
		GetDlgItemText(IDC_EDIT5, buf, sizeof(buf));
		if (jtransform_parse_crop_spec(&cropinfo, buf))
		{
			if (cropinfo.crop_width_set == JCROP_POS)
			{
				sprintf(buffer1, "%d", cropinfo.crop_width);
				SetDlgItemText(IDC_EDIT1, buffer1);
			}
			if (cropinfo.crop_height_set == JCROP_POS)
			{
				sprintf(buffer2, "%d", cropinfo.crop_height);
				SetDlgItemText(IDC_EDIT2, buffer2);
			}
			if (cropinfo.crop_xoffset_set == JCROP_POS)
			{
				sprintf(buffer3, "%d", cropinfo.crop_xoffset);
				SetDlgItemText(IDC_EDIT3, buffer3);
			}
			if (cropinfo.crop_yoffset_set == JCROP_POS)
			{
				sprintf(buffer4, "%d", cropinfo.crop_yoffset);
				SetDlgItemText(IDC_EDIT4, buffer4);
			}
		}
		GetDlgItem(IDC_EDIT1)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT2)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT3)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT4)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT5)->EnableWindow(FALSE);
	}
}

void CFrameDlg::OnRadio1()
{
	char buffer[128];
	int width;
	int height;
	int xoffset;
	int yoffset;
	width = ((CDibLookApp *)AfxGetApp())->m_width;
	height = ((CDibLookApp *)AfxGetApp())->m_height;
	xoffset = ((CDibLookApp *)AfxGetApp())->m_xoffset;
	yoffset = ((CDibLookApp *)AfxGetApp())->m_yoffset;
	sprintf(buffer, "%d", width);
	SetDlgItemText(IDC_EDIT1, buffer);
	sprintf(buffer, "%d", height);
	SetDlgItemText(IDC_EDIT2, buffer);
	if (xoffset < 0)
		buffer[0] = 0;
	else
		sprintf(buffer, "%d", xoffset);
	SetDlgItemText(IDC_EDIT3, buffer);
	if (yoffset < 0)
		buffer[0] = 0;
	else
		sprintf(buffer, "%d", yoffset);
	SetDlgItemText(IDC_EDIT4, buffer);
	if (xoffset < 0)
		sprintf(buffer, "%dx%d", width, height);
	else if (yoffset < 0)
		sprintf(buffer, "%dx%d+%d", width, height, xoffset);
	else
		sprintf(buffer, "%dx%d+%d+%d", width, height, xoffset, yoffset);
	SetDlgItemText(IDC_EDIT5, buffer);
}

void CFrameDlg::OnRadio2()
{
	char buffer[128];
	int width;
	int height;
	int xoffset;
	int yoffset;
	width = m_width;
	height = m_height;
	xoffset = m_xoffset;
	yoffset = m_yoffset;
	sprintf(buffer, "%d", width);
	SetDlgItemText(IDC_EDIT1, buffer);
	sprintf(buffer, "%d", height);
	SetDlgItemText(IDC_EDIT2, buffer);
	if (xoffset < 0)
		buffer[0] = 0;
	else
		sprintf(buffer, "%d", xoffset);
	SetDlgItemText(IDC_EDIT3, buffer);
	if (yoffset < 0)
		buffer[0] = 0;
	else
		sprintf(buffer, "%d", yoffset);
	SetDlgItemText(IDC_EDIT4, buffer);
	if (xoffset < 0)
		sprintf(buffer, "%dx%d", width, height);
	else if (yoffset < 0)
		sprintf(buffer, "%dx%d+%d", width, height, xoffset);
	else
		sprintf(buffer, "%dx%d+%d+%d", width, height, xoffset, yoffset);
	SetDlgItemText(IDC_EDIT5, buffer);
}

BEGIN_MESSAGE_MAP(CFrameDlg, CDialog)
	//{{AFX_MSG_MAP(CFrameDlg)
	ON_BN_CLICKED(IDC_CHECK1, OnCheck1)
	ON_BN_CLICKED(IDC_RADIO1, OnRadio1)
	ON_BN_CLICKED(IDC_RADIO2, OnRadio2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CDibView::OnPopup1DefineFrame()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	CFrameDlg frameDlg;
	frameDlg.m_width = pDoc->m_image_width - m_right - m_left;
	frameDlg.m_height = pDoc->m_image_height - m_bottom - m_top;
	frameDlg.m_xoffset = m_left;
	frameDlg.m_yoffset = m_top;
	if (frameDlg.DoModal() == IDOK)
	{
        refreshFrame ();
	}
}

void CDibView::OnDefinePhotoFrame()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

    myapp->m_xoffset = 0;
    myapp->m_yoffset = 0;
    int width = pDoc->m_image_width;
    int height = pDoc->m_image_height;

    if (width > height)
        {
        // landscape
        int newheight = width * 2 / 3;
        myapp->m_width = width;
        myapp->m_height = newheight;
        myapp->m_xoffset = 0;
        myapp->m_yoffset = (height - newheight) / 2;
        }
    else
        {
        // portrait
        int newwidth = height * 2 / 3;
        myapp->m_width = newwidth;
        myapp->m_height = height;
        myapp->m_xoffset = (width - newwidth) / 2;
        myapp->m_yoffset = 0;
        }

    TRACE ("photo frame: width %d, height %d, xoffset %d, yoffset %d\n", 
        myapp->m_width,
        myapp->m_height,
        myapp->m_xoffset,
        myapp->m_yoffset);
    refreshFrame ();
}

// refresh the frame
void CDibView::refreshFrame()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_xoffset >= 0 && myapp->m_xoffset < pDoc->m_image_width)
	{
		myapp->m_xoffset -= myapp->m_xoffset % pDoc->m_MCUwidth;
		m_left = myapp->m_xoffset;
	}
	if (myapp->m_yoffset >= 0 && myapp->m_yoffset < pDoc->m_image_height)
	{
		myapp->m_yoffset -= myapp->m_yoffset % pDoc->m_MCUheight;
		m_top = myapp->m_yoffset;
	}
	if (myapp->m_width <= 0 || myapp->m_width > pDoc->m_image_width)
		myapp->m_width = pDoc->m_image_width;
	m_right = pDoc->m_image_width - m_left - myapp->m_width;
	if (myapp->m_height <= 0 || myapp->m_height > pDoc->m_image_height)
		myapp->m_height = pDoc->m_image_height;
	m_bottom = pDoc->m_image_height - m_top - myapp->m_height;
	int lim_x = 0;
	int lim_y = 0;
	if (myapp->m_trim)
	{
		switch (pDoc->m_transform)
		{
		case 1:
		case 5:
			if (myapp->m_width > pDoc->m_image_width % pDoc->m_MCUwidth)
				lim_x = pDoc->m_image_width % pDoc->m_MCUwidth;
			break;
		case 2:
		case 7:
			if (myapp->m_height > pDoc->m_image_height % pDoc->m_MCUheight)
				lim_y = pDoc->m_image_height % pDoc->m_MCUheight;
			break;
		case 4:
		case 6:
			if (myapp->m_width > pDoc->m_image_width % pDoc->m_MCUwidth)
				lim_x = pDoc->m_image_width % pDoc->m_MCUwidth;
			if (myapp->m_height > pDoc->m_image_height % pDoc->m_MCUheight)
				lim_y = pDoc->m_image_height % pDoc->m_MCUheight;
			break;
		}
	}
	if (m_right < lim_x)
	{
		m_left += m_right - lim_x;
		if (m_left < 0) m_left = 0;
		m_right = lim_x + (m_left % pDoc->m_MCUwidth);
		m_left -= m_right - lim_x;
	}
	if (m_bottom < lim_y)
	{
		m_top += m_bottom - lim_y;
		if (m_top < 0) m_top = 0;
		m_bottom = lim_y + (m_top % pDoc->m_MCUheight);
		m_top -= m_bottom - lim_y;
	}

	char buf[64];
	unsigned int paru[2];
	int pari[2];
	sprintf(buf, " %dx%d+%d+%d",
		pDoc->m_image_width - m_right - m_left,
		pDoc->m_image_height - m_bottom - m_top, m_left, m_top);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.GetPaneInfo(1, paru[0], paru[1], pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_POS, SBPS_NORMAL, pari[0]);
	((CMainFrame *)AfxGetMainWnd())->m_wndStatusBar.SetPaneText(1, buf, TRUE);

	OnUpdate(NULL);
}


void CDibView::OnPopup1SaveFrameAs()
{
  GetDocument()->DoSave(NULL, FALSE);
}


void CDibView::OnViewZoom1()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 1) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			1, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 1;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 4);
	}
	else
	{
		if (m_zoomval == 1) return;
		OnUpdate(NULL, 4);
	}
}

void CDibView::OnViewZoom2()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 2) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			2, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 2;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 5);
	}
	else
	{
		if (m_zoomval == 2) return;
		OnUpdate(NULL, 5);
	}
}

void CDibView::OnViewZoom3()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 3) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			3, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 3;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 6);
	}
	else
	{
		if (m_zoomval == 3) return;
		OnUpdate(NULL, 6);
	}
}

void CDibView::OnViewZoom4()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 4) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			4, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 4;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 7);
	}
	else
	{
		if (m_zoomval == 4) return;
		OnUpdate(NULL, 7);
	}
}

void CDibView::OnViewZoom5()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 5) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			5, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 5;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 8);
	}
	else
	{
		if (m_zoomval == 5) return;
		OnUpdate(NULL, 8);
	}
}

void CDibView::OnViewZoom6()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 6) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			6, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 6;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 9);
	}
	else
	{
		if (m_zoomval == 6) return;
		OnUpdate(NULL, 9);
	}
}

void CDibView::OnViewZoom7()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 7) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			7, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 7;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 10);
	}
	else
	{
		if (m_zoomval == 7) return;
		OnUpdate(NULL, 10);
	}
}

void CDibView::OnViewZoom8()
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else
	{
		if (m_zoomval == 8) return;
		OnUpdate(NULL, 11);
	}
}

void CDibView::OnViewZoom9()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 9) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			9, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 9;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 12);
	}
	else
	{
		if (m_zoomval == 9) return;
		OnUpdate(NULL, 12);
	}
}

void CDibView::OnViewZoom10()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 10) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			10, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 10;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 13);
	}
	else
	{
		if (m_zoomval == 10) return;
		OnUpdate(NULL, 13);
	}
}

void CDibView::OnViewZoom11()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 11) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			11, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 11;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 14);
	}
	else
	{
		if (m_zoomval == 11) return;
		OnUpdate(NULL, 14);
	}
}

void CDibView::OnViewZoom12()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 12) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			12, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 12;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 15);
	}
	else
	{
		if (m_zoomval == 12) return;
		OnUpdate(NULL, 15);
	}
}

void CDibView::OnViewZoom13()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 13) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			13, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 13;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 16);
	}
	else
	{
		if (m_zoomval == 13) return;
		OnUpdate(NULL, 16);
	}
}

void CDibView::OnViewZoom14()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 14) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			14, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 14;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 17);
	}
	else
	{
		if (m_zoomval == 14) return;
		OnUpdate(NULL, 17);
	}
}

void CDibView::OnViewZoom15()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 15) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			15, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 15;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 18);
	}
	else
	{
		if (m_zoomval == 15) return;
		OnUpdate(NULL, 18);
	}
}

void CDibView::OnViewZoom16()
{
	CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
	CDibDoc *pDoc = GetDocument();

	if (myapp->m_zoom_reload)
	{
		if (pDoc->m_scale == 16) return;
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			16, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 16;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 11);
	}
	else if (pDoc->m_scale != 8)
	{
		BeginWaitCursor();
		pDoc->ReplaceHDIB(NULL);

		HDIB hDIB = (HDIB) Read_JPEG_File(pDoc->GetPathName(),
			8, pDoc->m_force_grayscale,
			&pDoc->m_image_width, &pDoc->m_image_height,
			&pDoc->m_MCUwidth, &pDoc->m_MCUheight);

		pDoc->ReplaceHDIB(hDIB);
		pDoc->m_dithered = 0;
		SmartDither();
		pDoc->m_scale = 8;
		pDoc->DoTransform();
		pDoc->InitDIBData();
		EndWaitCursor();
		pDoc->UpdateAllViews(NULL, 19);
	}
	else
	{
		if (m_zoomval == 16) return;
		OnUpdate(NULL, 19);
	}
}

void CDibView::OnUpdateViewZoom1(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 1);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 1);
	}
}

void CDibView::OnUpdateViewZoom2(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 2);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 2);
	}
}

void CDibView::OnUpdateViewZoom3(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 3);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 3);
	}
}

void CDibView::OnUpdateViewZoom4(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 4);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 4);
	}
}

void CDibView::OnUpdateViewZoom5(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 5);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 5);
	}
}

void CDibView::OnUpdateViewZoom6(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 6);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 6);
	}
}

void CDibView::OnUpdateViewZoom7(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 7);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 7);
	}
}

void CDibView::OnUpdateViewZoom8(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 8);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 8);
	}
}

void CDibView::OnUpdateViewZoom9(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 9);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 9);
	}
}

void CDibView::OnUpdateViewZoom10(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 10);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 10);
	}
}

void CDibView::OnUpdateViewZoom11(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 11);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 11);
	}
}

void CDibView::OnUpdateViewZoom12(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 12);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 12);
	}
}

void CDibView::OnUpdateViewZoom13(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 13);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 13);
	}
}

void CDibView::OnUpdateViewZoom14(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 14);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 14);
	}
}

void CDibView::OnUpdateViewZoom15(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 15);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 15);
	}
}

void CDibView::OnUpdateViewZoom16(CCmdUI *pCmdUI)
{
	CDibDoc *pDoc = GetDocument();

	if (pDoc->m_scale != 8)
	{
		pCmdUI->SetCheck(pDoc->m_scale == 16);
	}
	else
	{
		pCmdUI->SetCheck(m_zoomval == 16);
	}
}

void CDibView::OnUpdateToolbarZoom(CCmdUI *pCmdUI)
{
	int scale = GetDocument()->m_scale;
	if (scale == 8) scale = m_zoomval;
	(((CMainFrame *)AfxGetApp()->m_pMainWnd)->m_slider).SetPos(scale);

	switch (scale)
	{
	case  1: pCmdUI->SetText("12.5 %");  break;
	case  2: pCmdUI->SetText("25 %");    break;
	case  3: pCmdUI->SetText("37.5 %");  break;
	case  4: pCmdUI->SetText("50 %");    break;
	case  5: pCmdUI->SetText("62.5 %");  break;
	case  6: pCmdUI->SetText("75 %");    break;
	case  7: pCmdUI->SetText("87.5 %");  break;
	case  8: pCmdUI->SetText("100 %");   break;
	case  9: pCmdUI->SetText("112.5 %"); break;
	case 10: pCmdUI->SetText("125 %");   break;
	case 11: pCmdUI->SetText("137.5 %"); break;
	case 12: pCmdUI->SetText("150 %");   break;
	case 13: pCmdUI->SetText("162.5 %"); break;
	case 14: pCmdUI->SetText("175 %");   break;
	case 15: pCmdUI->SetText("187.5 %"); break;
	case 16: pCmdUI->SetText("200 %");   break;
	}
}
